/* Copyright 1999 by Donald Reble. All rights reserved. */

/* $Id: $ */
/* This class represents an effect. */

package cgmud.effect;

import java.awt.Component;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Polygon;
import java.awt.Toolkit;
import cgmud.base.ByteSequence;


public abstract class Effect {

    public static final byte
	ELSE		   =  1,
	FI		   =  2,
	IF_FOUND	   =  3,
	FAIL_TEXT	   =  4,
	ABORT		   =  5,
	CALL		   =  6,
	PLACE_CURSOR	   =  7,
	PLACE_CURSOR_SHORT =  8,
	REMOVE_CURSOR	   =  9,
	SET_CURSOR_PEN	   = 10,
	SET_CURSOR_PATTERN = 11,
	ADD_BUTTON	   = 12,
	ERASE_BUTTON	   = 13,
	CLEAR_BUTTONS	   = 14,
	ADD_REGION	   = 15,
	ERASE_REGION	   = 16,
	CLEAR_REGIONS	   = 17,
	SET_BUTTON_PEN	   = 18,
	SET_PEN		   = 19,
	SET_COLOUR	   = 20,
	RESET_COLOURS	   = 21,
	CLEAR		   = 22,
	PIXEL		   = 23,
	PIXEL_ROW	   = 24,
	ABS_MOVE	   = 25,
	ABS_MOVE_SHORT	   = 26,
	REL_MOVE	   = 27,
	REL_MOVE_SHORT	   = 28,
	ABS_DRAW	   = 29,
	ABS_DRAW_SHORT	   = 30,
	REL_DRAW	   = 31,
	REL_DRAW_SHORT	   = 32,
	RECTANGLE	   = 33,
	CIRCLE		   = 34,
	ELLIPSE		   = 35,
	POLYGON_START	   = 36,
	POLYGON_END	   = 37,
	DEFINE_TILE	   = 38,
	DEFINE_OVERLAY	   = 39,
	DISPLAY_TILE	   = 40,
	SET_TEXT_COLOUR    = 41,
	TEXT		   = 42,
	LOAD_BACKGROUND    = 43,
	SET_IMAGE	   = 44,
	SHOW_IMAGE	   = 45,
	SHOW_IMAGE_PIXELS  = 46,
	SHOW_BRUSH	   = 47,
	SCROLL_RECTANGLE   = 48,
	SET_ICON_PEN	   = 49,
	NEW_ICON	   = 50,
	SHOW_ICON	   = 51,
	REMOVE_ICON	   = 52,
	DELETE_ICON	   = 53,
	RESET_ICONS	   = 54,
	SOUND_VOLUME	   = 55,
	PLAY_SOUND	   = 56,
	PARAMS		   = 57,
	V_RESET		   = 58,
	VOICE_VOLUME	   = 59,
	NARRATE		   = 60,
	MUSIC_VOLUME	   = 61,
	PLAY_SONG	   = 62;

    protected static boolean d_foundFlag = false;
    protected static Image d_theImage = null;
    protected static int d_ptX, d_ptY;
    protected static Polygon d_polygon;
    protected static int
	d_iconPen = 0,			// black
	d_buttonBackground = 254,	// light grey
	d_buttonForeground = 0;		// black


    public abstract void run(Component com) throws EffectException;

    public static Effect extract(ByteSequence bs)
    throws EffectException
    {
	try {
	    int kind = bs.getByte();
	    switch (kind) {
		case ELSE:
		    // should have been grabbed by IF_FOUND
		    throw new EffectFormException("unmatched ELSE");
		case FI:
		    // should have been grabbed by IF_FOUND
		    throw new EffectFormException("unmatched FI");
		case POLYGON_END:
		    // should have been grabbed by POLYGON_START
		    throw new EffectFormException("unmatched POLYGON_END");

		case IF_FOUND:           return new IfFound(bs);
		case FAIL_TEXT:          return new FailText(bs);
		case ABORT:              return new Abort(bs);
		case CALL:               return new Call(bs);
		case PLACE_CURSOR:       return new PlaceCursor(bs, true);
		case PLACE_CURSOR_SHORT: return new PlaceCursor(bs, false);
		case REMOVE_CURSOR:      return new RemoveCursor(bs);
		case SET_CURSOR_PEN:     return new SetCursorPen(bs);
		case SET_CURSOR_PATTERN: return new SetCursorPattern(bs);
		case ADD_BUTTON:         return new AddButton(bs);
		case ERASE_BUTTON:       return new EraseButton(bs);
		case CLEAR_BUTTONS:      return new ClearButtons(bs);
		case ADD_REGION:         return new AddRegion(bs);
		case ERASE_REGION:       return new EraseRegion(bs);
		case CLEAR_REGIONS:      return new ClearRegions(bs);
		case SET_BUTTON_PEN:     return new SetButtonPen(bs);
		case SET_PEN:            return new SetPen(bs);
		case SET_COLOUR:         return new SetColour(bs);
		case RESET_COLOURS:      return new ResetColours(bs);
		case CLEAR:              return new Clear(bs);
		case PIXEL:              return new Pixel(bs);
		case PIXEL_ROW:          return new PixelRow(bs);
		case ABS_MOVE:           return new AbsMove(bs, true);
		case ABS_MOVE_SHORT:     return new AbsMove(bs, false);
		case REL_MOVE:           return new RelMove(bs, true);
		case REL_MOVE_SHORT:     return new RelMove(bs, false);
		case ABS_DRAW:           return new AbsDraw(bs, true);
		case ABS_DRAW_SHORT:     return new AbsDraw(bs, false);
		case REL_DRAW:           return new RelDraw(bs, true);
		case REL_DRAW_SHORT:     return new RelDraw(bs, false);
		case RECTANGLE:          return new Rectangle(bs);
		case CIRCLE:             return new Circle(bs);
		case ELLIPSE:            return new Ellipse(bs);
		case POLYGON_START:      return new PolygonStart(bs);
		case DEFINE_TILE:        return new DefineTile(bs, false);
		case DEFINE_OVERLAY:     return new DefineTile(bs, true);
		case DISPLAY_TILE:       return new DisplayTile(bs);
		case SET_TEXT_COLOUR:    return new SetTextColour(bs);
		case TEXT:               return new Text(bs);
		case LOAD_BACKGROUND:    return new LoadBackground(bs);
		case SET_IMAGE:          return new SetImage(bs);
		case SHOW_IMAGE:         return new ShowImage(bs);
		case SHOW_IMAGE_PIXELS:  return new ShowImagePixels(bs);
		case SHOW_BRUSH:         return new ShowBrush(bs);
		case SCROLL_RECTANGLE:   return new ScrollRectangle(bs);
		case SET_ICON_PEN:       return new SetIconPen(bs);
		case NEW_ICON:           return new NewIcon(bs);
		case SHOW_ICON:          return new ShowIcon(bs);
		case REMOVE_ICON:        return new RemoveIcon(bs);
		case DELETE_ICON:        return new DeleteIcon(bs);
		case RESET_ICONS:        return new ResetIcons(bs);
		case SOUND_VOLUME:       return new SoundVolume(bs);
		case PLAY_SOUND:         return new PlaySound(bs);
		case PARAMS:             return new Params(bs);
		case V_RESET:            return new VReset(bs);
		case VOICE_VOLUME:       return new VoiceVolume(bs);
		case NARRATE:            return new Narrate(bs);
		case MUSIC_VOLUME:       return new MusicVolume(bs);
		case PLAY_SONG:          return new PlaySong(bs);
	    }
	    throw new EffectFormException("unknown effect kind " + kind);
	} catch (ArrayIndexOutOfBoundsException ex) {
	    throw new EffectFormException("EOS while extracting effect");
	}
    }

    public static Image fetchImage(String filename, Component drawon) {
	StringBuffer sb = new StringBuffer(
	    System.getProperty("ImageDir", "")
	);
	sb.append("/");
	sb.append(filename);
	// System.out.println("File: " + sb.toString());
	MediaTracker mt = new MediaTracker(drawon);
	Image im = Toolkit.getDefaultToolkit().getImage(sb.toString());
	mt.addImage(im, 0);
	try {
	    mt.waitForAll();
	} catch (InterruptedException ex) {
	    d_foundFlag = false;
	// System.out.println(sb.toString() + " aborted");
	    return null;
	}
	d_foundFlag = ! mt.isErrorAny();
	// System.out.println(sb.toString() + ": " + mt.statusID(0, false));
	return im;
    }

}
