/* Copyright 1999 by Donald Reble. All rights reserved. */

/* $Id: $ */
/* This class does message-logging. */

package cgmud.main;

import java.io.FileWriter;
import java.io.IOException;
import cgmud.base.ByteSequence;


public class LogFile {

    private FileWriter d_file;


    /* ----- constructors ----- */

    public LogFile(String filename) {
	d_file = null;
	if (filename != null) {
	    try {
		d_file = new FileWriter(filename);
	    } catch (IOException ex) {
		System.err.println("LogFile: " + ex.toString());
	    }
	}
    }


    /* ----- other methods ----- */

    private void m_log(String str) {
	if (d_file == null) return;
	try {
	    d_file.write(str, 0, str.length());
	    d_file.flush();
	} catch (IOException ex) {
	    System.err.println("LogFile: " + ex.toString());
	    d_file = null;
	}
    }

    public void logInput(ByteSequence bs) {
	if (d_file != null) {
	    m_log(LogFormatter.formatByteSequence('i', bs));
	}
    }

    public void logOutput(ByteSequence bs) {
	if (d_file != null) {
	    m_log(LogFormatter.formatByteSequence('o', bs));
	}
    }

    public void logExceptional(ByteSequence bs) {
	if (d_file != null) {
	    m_log(LogFormatter.formatByteSequence('x', bs));
	}
    }

    public void logString(String str) {
	if (d_file != null) m_log(str);
    }

    public void logStringAlways(String str) {
	if (d_file != null) {
	    m_log(str + '\n');
	} else {
	    System.err.println(str);
	}
    }

    public boolean isLogging() { return d_file != null; }

    public void close() {
	if (d_file == null) return;
	try {
	    d_file.close();
	} catch (IOException ex) {
	    System.err.println("LogFile: " + ex.toString());
	}
	d_file = null;
    }

}
