/* Copyright 1999 by Donald Reble. All rights reserved. */

/* $Id: $ */
/* This is the main class of cgMUD. */

package cgmud.main;

import java.awt.*;
import java.awt.event.*;
import java.net.Socket;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import cgmud.effect.EffectsInfo;
import cgmud.effect.Region;
import cgmud.event.*;
import cgmud.message.*;
import cgmud.ui.*;


public class Main implements ActionListener, MouseListener {

    private ThreadedReader d_serverIn;

    private boolean d_userquit;

    /* ----- UI ----- */

    private static final short
	D_TEXT_ROWS = 30,
	D_TEXT_COLS = 80;

    private BoxFrame d_newframe;


    /* ----- player/password ----- */
    private boolean d_needCreatePW;
    private int d_pwstate;
	private static final int D_PWSTATE_NORMAL    = 0;
	private static final int D_PWSTATE_GET       = 1;
	private static final int D_PWSTATE_MAKE      = 2;
	private static final int D_PWSTATE_CONFIRM   = 3;
	private static final int D_PWSTATE_REGET     = 4;
	private static final int D_PWSTATE_REMAKE    = 5;
	private static final int D_PWSTATE_RECONFIRM = 6;
    private int d_triesleft;
    private String d_firstPW;

    private String d_playername;


    /* ----- main ----- */

    public static void main(String[] args) {
	String usage =
	   "Usage: Main [-s<fontsize>] [-f] [-2] inetaddr port [logfile]";
	int argCnt, argNum, textRows, fontSize, port;
	String arg, flag, logfile;
	boolean refocus, doTwice;

	textRows = D_TEXT_ROWS;
	fontSize = 7;
	refocus = false;
	doTwice = false;
	argCnt = args.length;
	argNum = 0;
	while (argNum < argCnt) {
	    arg = args[argNum];
	    if (! arg.substring(0, 1).equals("-")) {
		break;
	    }
	    arg = arg.substring(1, arg.length());
	    flag = arg.substring(0, 1);
	    arg = arg.substring(1, arg.length());
	    if (flag.equals("t")) {
		try {
		    textRows = Integer.parseInt(arg);
		} catch (NumberFormatException ex) {
		    System.err.println(arg + " isn't numeric");
		    return;
		}
	    } else if (flag.equals("s")) {
		try {
		    fontSize = Integer.parseInt(arg);
		} catch (NumberFormatException ex) {
		    System.err.println(arg + " isn't numeric");
		    return;
		}
	    } else if (flag.equals("f")) {
		refocus = true;
	    } else if (flag.equals("2")) {
		doTwice = true;
	    } else {
		System.err.println("Unknown flag '" + flag + "'");
		System.err.println(usage);
		return;
	    }
	    argNum += 1;
	}
	if (argCnt < argNum + 2) {
	    System.err.println(usage);
	    return;
	}
	try {
	    port = Integer.parseInt(args[argNum + 1]);
	} catch (NumberFormatException ex) {
	    System.err.println(args[argNum + 1] + " isn't numeric");
	    return;
	}

	Main main = new Main(textRows, fontSize, refocus, args[argNum], port,
	    (argCnt > argNum + 2) ? args[argNum + 2] : null, doTwice);
    }


    /* ----- constructor ----- */

    public Main(int textRows, int fontSize, boolean refocus,
		String inetaddr, int port, String logfile, boolean doTwice)
    {
	Globals.make(this, textRows, fontSize, refocus);
	d_userquit = false;

	m_makeUI(doTwice);
	d_newframe = null;

	try {
	    Socket sock = new Socket(inetaddr, port);
	    d_serverIn = new ThreadedReader(sock.getInputStream(),
					    Globals.theFrame);
	    Globals.serverOut = new BufferedOutputStream(
		sock.getOutputStream()
	    );
	} catch (IOException ex) {
	    System.err.println(ex.toString());
	    System.err.println("can't connect to " + inetaddr + ' ' + port);
	    System.exit(1);
	}

	Globals.logFile = new LogFile(logfile);
	    // That makes a no-op LogFile if <logfile> is null.

	m_sendBeginClient();
	Globals.sessionState = Globals.SESSION_MSG_BEGIN_CLIENT;
	d_pwstate = D_PWSTATE_NORMAL;
    }


    /* ----- ActionListener methods ----- */

    public void actionPerformed(ActionEvent ev) {
	String command = ev.getActionCommand();
	if (command.equals("Quit")) {
	    d_userquit = true;
	    m_shutDown();
	    printdiag("Shutting down.");
	    return;
	}
	if (command.startsWith("Button#")) {
	    try {
		int butid = Integer.parseInt(command.substring(7));
		sendMessage(new ButtonHit(butid));
	    } catch (StringIndexOutOfBoundsException ex) {
		// The string is exactly "Button#".
		printdiag("Internal error -- truncated Button");
	    } catch (NumberFormatException ex) {
		// The string is malformed in some other way.
		printdiag("Internal error -- misformed Button");
	    }
	}
    }


    /* ----- event processing ----- */

    public void handleRawKey(int code) {
	if (Globals.sessionState == Globals.SESSION_PLAYING) {
	    sendMessage(new RawKey(code));
	}
    }

    public void processThreadEndEvent(ThreadEndEvent te) {
	// connection closed
	d_serverIn = null;
	m_shutDown();
	printdiag("Connection to server closed.");
    }

    public void processMessageEvent(MessageEvent me) {
	Message msg = me.getMessage();
	Globals.logFile.logInput(msg);
	msg.process(this);
    }

    public void processLineEvent(LineEvent le) {
	String line = le.getLine();
	boolean passwdOK = false;

	switch (d_pwstate) {

	    case D_PWSTATE_GET:
//		setPrompt("");
		d_pwstate = D_PWSTATE_NORMAL;
		break;

	    case D_PWSTATE_MAKE:
		d_firstPW = line;
		if (line.equals("")) {
		    /* Empty password */
		    printdiag("Empty password - creation cancelled.");
		    d_pwstate = D_PWSTATE_NORMAL;
		    setPrompt("Enter character name:");
		    Globals.sessionState = Globals.SESSION_PLAYER_NAME;
		    return;
		}
		setBlanking(true);
		setPrompt("Confirm character password:");
		d_pwstate = D_PWSTATE_CONFIRM;
		return;

	    case D_PWSTATE_CONFIRM:
		passwdOK = line.equals(d_firstPW);
		d_firstPW = null;
		if (! passwdOK) {
		    printdiag("Password confirmation failed.");
		    setBlanking(true);
		    setPrompt("Reenter character password:");
		    d_pwstate = D_PWSTATE_MAKE;
		    return;
		}
//		setPrompt("");
		d_pwstate = D_PWSTATE_NORMAL;
		break;

	    case D_PWSTATE_REGET:
		sendMessage(new ChangePassword(line));
//		setPrompt("");
		d_pwstate = D_PWSTATE_NORMAL;
		return;

	    case D_PWSTATE_REMAKE:
		d_firstPW = line;
		setBlanking(true);
		setPrompt("Confirm new password:");
		d_pwstate = D_PWSTATE_RECONFIRM;
		return;

	    case D_PWSTATE_RECONFIRM:
		passwdOK = line.equals(d_firstPW);
		d_firstPW = null;
		if (! passwdOK) {
		    printdiag("Password confirmation failed.");
		    sendMessage(new ChangePassword(null));
		} else {
		    printdiag("Password changed.");
		    sendMessage(new ChangePassword(line));
		}
//		setPrompt("");
		d_pwstate = D_PWSTATE_NORMAL;
		return;

	    default:
		/* fallout */
	}

	switch (Globals.sessionState) {
	    case Globals.SESSION_PLAYER_NAME:
//		setPrompt("");
		d_playername = line.trim();
		if (line.equals("")) {
		    stopConnect();
		    break;
		}
		sendMessage(new AskPlayer(d_playername));
		Globals.sessionState = Globals.SESSION_MSG_ASK_PLAYER;
		break;
	    case Globals.SESSION_CONFIRM_NEWPLAYER:
		String answer = line.trim().toLowerCase();
		if (answer.equals("yes") || answer.equals("y")) {
		    if (d_needCreatePW) {
			Globals.sessionState = Globals.SESSION_CREATION_PW;
			requestPassword(false);
		    } else {
			Globals.sessionState = Globals.SESSION_NEWPLAYER_PW;
			makePassword();
		    }
		} else if (answer.equals("no") || answer.equals("n")) {
		    requestPlayerName();
		} else {
		    println("Answer \"yes\" or \"no\".");
		}
		break;
	    case Globals.SESSION_CREATION_PW:
		sendMessage(new CreationCheck(line));
		Globals.sessionState = Globals.SESSION_MSG_CREATION_CHECK;
		break;
	    case Globals.SESSION_NEWPLAYER_PW:
		sendMessage(new CreatePlayer(d_playername, line));
		Globals.sessionState = Globals.SESSION_MSG_CREATE_PLAYER;
		break;
	    case Globals.SESSION_OLDPLAYER_PW:
		sendMessage(new LoginPlayer(line));
		Globals.sessionState = Globals.SESSION_MSG_LOGIN_PLAYER;
		break;
	    case Globals.SESSION_PLAYING:
		/* client isn't expecting anything special */
		m_appendLine(line, true);
		sendMessage(new InputLine(line));
		break;
	    case Globals.SESSION_SHUTDOWN:
		/* We are going away. */
		System.exit(0);
		break;
	    default:
		/* ignore -- server isn't ready */
		break;
	}
    }


    /* ----- MouseListener interface ----- */

    public void mouseClicked(MouseEvent me) { }
    public void mouseEntered(MouseEvent me) { }
    public void mouseExited(MouseEvent me) { }
    public void mouseReleased(MouseEvent me) { }

    public void mousePressed(MouseEvent me) {
	if (Globals.sessionState != Globals.SESSION_PLAYING) return;
	Point pt = me.getPoint();
	Rectangle rect = null;
	int lowestid = 0;
	for (Enumeration ids = Globals.theRegions.keys();
	     ids.hasMoreElements();
	    )
	{
	    int id = ((Integer) (ids.nextElement())).intValue();
	    Region reg = (Region) Globals.theRegions.getItem(id);
	    Rectangle rr = reg.getRectangle();
	    if ((me.getComponent() == reg.getComponent())
		&& rr.contains(pt)
		&& ((rect == null) || (lowestid > id)))
	    {
		rect = rr;
		lowestid = id;
	    }
	}
	if (rect != null) {
	    pt.translate(-rect.x, -rect.y);
	    sendMessage(new RegionSelect(lowestid, pt));
	}
    }


    /* ----- transactions ----- */

    private boolean m_connected() {
	return (Globals.serverOut != null) 
	    && (d_serverIn != null);
    }

    public void sendMessage(Message msg) {
	if (m_connected()) {
	    Globals.logFile.logOutput(msg);
	    try {
		msg.send(Globals.serverOut);
	    } catch (IOException ie) {
		printdiag("IOexception: " + ie.toString());
		m_shutDown();
	    }
	}
    }

    private void m_sendBeginClient() {
	EffectsInfo ei = new EffectsInfo();
	Dimension scrsize = Toolkit.getDefaultToolkit().getScreenSize();
	System.arraycopy(
	    "Java 1.1 AWT        ".getBytes(), 0,
	    ei.graphicsType, 0, ei.graphicsType.length);
	ei.screenRows = (short) scrsize.width;
	ei.screenCols = (short) scrsize.height;
	ei.screenColours = (short) PenSet.PEN_QUAN;
	ei.fontAscent = (short) Globals.theFontMetrics.getAscent();
	ei.fontDescent = (short) Globals.theFontMetrics.getDescent();
	ei.fontLeading = (short) Globals.theFontMetrics.getLeading();
	ei.fontHeight = (short) Globals.theFontMetrics.getHeight();
//	ei.fontWidth = (short) Globals.theFontMetrics.getMaxAdvance();
	ei.fontWidth = (short) Globals.theFontMetrics.charWidth('M');
	ei.textWidth = (short) D_TEXT_COLS;
	ei.textHeight = (short) Globals.textRows;
	ei.version = 0;
	ei.canEdit = false;
	ei.canGetString = true;
	ei.canQueryFile = false;
	ei.canWizard = false;
	ei.graphicsOn = true;
	ei.graphicsPalette = true;
	ei.voiceOn = false;
	ei.soundOn = false;
	ei.musicOn = false;
	sendMessage(new BeginClient(ei));
    }

    public void stopConnect() {
	setPrompt("Press RETURN:");
	m_shutDown();
    }

    public void requestPlayerName() {
	Globals.sessionState = Globals.SESSION_PLAYER_NAME;
	setPrompt("Enter character name:");
    }

    public void confirmPlayerCreation(boolean needCreatePW) {
	d_needCreatePW = needCreatePW;
	Globals.sessionState = Globals.SESSION_CONFIRM_NEWPLAYER;
	println("Character " + d_playername + " does not exist...");
	setPrompt("Create it (yes/no)?");
    }

    public void requestPassword(boolean player) {
	d_pwstate = D_PWSTATE_GET;
	if (player) {
	    setPrompt("Enter character password:");
	} else {
	    setPrompt("Enter creation password:");
	}
	setBlanking(true);
    }

    public void makePassword() {
	d_pwstate = D_PWSTATE_MAKE;
	d_firstPW = null;
	setPrompt("Enter character password:");
	setBlanking(true);
    }

    public void remakePassword(boolean player, boolean newpw) {
	setBlanking(true);
	setPrompt((newpw ? "New" : "Old") +
		  (player ? " player" : " creation") +
		  " password:");
	d_pwstate = newpw ? D_PWSTATE_REMAKE : D_PWSTATE_REGET;
    }

    public void abandonConnect() {
	sendMessage(new EndClient());
	Globals.sessionState = Globals.SESSION_MSG_END_CLIENT;
    }

    public void disconnectNow(String why) {
	printdiag(why);
	m_shutDown();
    }

    public void serverError() {
	disconnectNow("Communication error: report to adminstrator.");
	try {
	    throw new Exception();
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    private void m_shutDown() {
	Globals.sessionState = Globals.SESSION_SHUTDOWN;
	if (Globals.logFile.isLogging()) {
	    Globals.logFile.close();
	}
	if (Globals.serverOut != null) {
	    try {
		Globals.serverOut.close();
	    } catch (IOException ix) {
		System.err.println("IOException closing serverOut.");
	    }
	    Globals.serverOut = null;
	}
	if (d_serverIn != null) {
	    d_serverIn.stop();
	    // wait for ThreadEndEvent
	} else if (d_userquit) {
	    System.exit(0);
	}
    }

    public void waitForRunClient() {
	Globals.sessionState = Globals.SESSION_MSG_RUN_CLIENT;
    }


    /* ----- UI ----- */

    private Font m_makeFixedFont(int target) {
	// Find the smallest DialogInput font with maxAdvance >= target
	Toolkit tk = Toolkit.getDefaultToolkit();
	Font font;
	FontMetrics fm;
	// At least on my system, even sizes look much better!
	for (int points = 2; ; points += 2) {
	    font = new Font("DialogInput", Font.PLAIN, points);
	    fm = tk.getFontMetrics(font);
	    if (fm.charWidth('M') >= target) {
		return font;
	    }
	}
    }

    private void m_addMenuItem(Menu menu, String itemname) {
	MenuItem mi = new MenuItem(itemname);
	mi.addActionListener(this);
	menu.add(mi);
    }

    private MenuBar m_makeMenus() {
	MenuBar menubar = new MenuBar();
	Menu filemenu = new Menu("File", false);
	m_addMenuItem(filemenu, "Quit");
	menubar.add(filemenu);
	return menubar;
    }

    private void m_makeUI(boolean doTwice) {
	Globals.theFixedFont = m_makeFixedFont(Globals.fontSize);
	Globals.theFontMetrics = Toolkit.getDefaultToolkit().
	    getFontMetrics(Globals.theFixedFont);

	/* On startup, we have only a GeneralText and a PromptedText. */
	BoxFrame bf = new BoxFrame();
	bf.setFont(Globals.theFixedFont);
	bf.addContainer(0, 0, true, BoxFrame.ALIGN_MED);
	bf.addComponent(0, 1, BoxFrame.GENERAL_TEXT,
			D_TEXT_COLS, (short) Globals.textRows);
	bf.addComponent(0, 2, BoxFrame.PROMPTED_TEXT,
			D_TEXT_COLS, (short) 1);
	bf.setMenuBar(m_makeMenus());
	bf.setVisible(false);
	bf.makeFrame();
	Globals.theFrame = bf;
	Globals.theDefaultPrompter = bf.getDefaultPromptedText();
	Globals.theDefaultText     = bf.getDefaultGeneralText();
	bf.addKeyListener(Globals.theDefaultPrompter);

	if (doTwice) {
	    /* Do it a second time to try to make it work on later JDK's that
	       seem to get the sizes somewhat messed up. */
	    BoxFrame bf2 = new BoxFrame();
	    bf2.setFont(Globals.theFixedFont);
	    bf2.addContainer(0, 0, true, BoxFrame.ALIGN_MED);
	    bf2.addComponent(0, 1, BoxFrame.GENERAL_TEXT,
			     D_TEXT_COLS, (short) Globals.textRows);
	    bf2.addComponent(0, 2, BoxFrame.PROMPTED_TEXT,
			     D_TEXT_COLS, (short) 1);
	    bf2.setMenuBar(m_makeMenus());
	    bf2.setVisible(false);
	    bf2.makeFrame();
	    Globals.theFrame = bf2;
	    Globals.theDefaultPrompter = bf2.getDefaultPromptedText();
	    Globals.theDefaultText     = bf2.getDefaultGeneralText();
	    bf2.addKeyListener(Globals.theDefaultPrompter);

	    /* Only now can we dispose of the first one. Sigh. */
	    bf.dispose();
	}
    }


    /* ----- string requester methods ----- */

    public void requestString(String label, String def) {
	new StringRequester(this, label, def);
    }

    public void stringCancelled() {
	sendMessage(new GetStringDone(false, ""));
    }

    public void stringGiven(String str) {
	sendMessage(new GetStringDone(true, str));
    }


    /* ----- line output ----- */

    private void m_appendLine(String line, boolean withPrompt) {
	if (withPrompt) {
	    line = Globals.theDefaultPrompter.getPrompt() + line;
	}
	if (Globals.theDefaultText != null) {
	    Globals.theDefaultText.appendLine(line);
	} else {
	    System.out.println(line);
	}
    }

    public void println(String line) {
	m_appendLine(line, false);
    }

    public void printdiag(String line) {
	m_appendLine("<<< " + line + " >>>", false);
    }


    /* ----- prompter control ----- */

    public void setBlanking(boolean blank) {
	if (Globals.theDefaultPrompter != null) {
	    Globals.theDefaultPrompter.setBlanking(blank);
	}
    }

    public void setPrompt(String prompt) {
	if (Globals.theDefaultPrompter != null) {
	    Globals.theDefaultPrompter.setPrompt(prompt);
	} else {
	    System.out.println(prompt);
	}
    }


    /* ----- frame builders ----- */

    public void addComponent(int encloser, int id, short kind,
			     short width, short height)
    {
	d_newframe.addComponent(encloser, id, kind, width, height);
    }

    public void addContainer(int encloser, int id, boolean vertical,
			     short alignment)
    {
	if (d_newframe == null) {
	    d_newframe = new BoxFrame();
	}
	d_newframe.addContainer(encloser, id, vertical, alignment);
    }

    public void makeFrame() {
	d_newframe.setMenuBar(m_makeMenus());
	d_newframe.makeFrame();
	Globals.theFrame.setVisible(false);
//%%%	Globals.theFrame.dispose();
	Globals.theFrame = d_newframe;
	Globals.theDefaultPrompter = d_newframe.getDefaultPromptedText();
	Globals.theDefaultText     = d_newframe.getDefaultGeneralText();
	d_newframe = null;
    }


    /* ----- inspectors ----- */

    public String getPlayerName() { return d_playername; }

}
