/* Copyright 1999 by Donald Reble. All rights reserved. */

/* $Id: $ */
/* This class represents an ask-player message. */

package cgmud.message;

import cgmud.base.ByteSequence;
import cgmud.main.Globals;
import cgmud.main.Main;


public class AskPlayer extends Message {

    /* result codes from server */
    public static final int RC_NOSUCH    = 0;
    public static final int RC_NEWPLAYER = 1;
    public static final int RC_TRYCREATE = 2;
    public static final int RC_NOTPLAYER = 3;
    public static final int RC_MISADMIN  = 4;
    public static final int RC_OK        = 5;
    public static final int RC_ALREADY   = 6;


    /* ----- constructors ----- */

    public AskPlayer(String player) {
	super(Message.ASK_PLAYER, 0, (byte) 0,
	    (short) (player.length() + 1));
	setIndex(D_LOC_TAIL);
	appendString(player);
    }

    /* packaged */ AskPlayer(ByteSequence data) { super(data); }


    /* ----- processing ----- */

    public void process(Main main) {
	if (Globals.sessionState == Globals.SESSION_SHUTDOWN) return;
	if (Globals.sessionState != Globals.SESSION_MSG_ASK_PLAYER) {
	    main.serverError();
	    return;
	}
	switch (getKey()) {
	    case RC_NOSUCH:
		main.println("Character " + main.getPlayerName() +
		    " does not exist; character creation is disallowed.");
		main.requestPlayerName();
		break;
	    case RC_NEWPLAYER:
		main.confirmPlayerCreation(false);
		return;
	    case RC_TRYCREATE:
		main.confirmPlayerCreation(true);
		return;
	    case RC_NOTPLAYER:
		main.println("You may not use that name.");
		main.requestPlayerName();
		return;
	    case RC_MISADMIN:
		main.println("Remote sysadmin logins are forbidden.");
		break;
	    case RC_OK:
		Globals.sessionState = Globals.SESSION_OLDPLAYER_PW;
		main.requestPassword(true);
		return;
	    case RC_ALREADY:
		main.println("You are already logged in.");
		break;
	    default:
		main.serverError();
		break;
	}
	main.abandonConnect();
    }

}
