/* Copyright 1999 by Donald Reble. All rights reserved. */

/* $Id: $ */
/* This class represents a begin-client message. */

package cgmud.message;

import cgmud.base.ByteSequence;
import cgmud.effect.EffectsInfo;
import cgmud.main.Globals;
import cgmud.main.Main;


public class BeginClient extends Message {

    /* result codes from server */
    public static final int
	RC_DENIED   = 0,
	RC_OKSINGLE = 1,
	RC_OK       = 2;


    /* ----- constructors ----- */

    public BeginClient(EffectsInfo ei) {
	super(Message.BEGIN_CLIENT, 0, (byte) 0,
	    (short) EffectsInfo.BYTE_SIZE);
	setIndex(D_LOC_TAIL);
	appendBlock(EffectsInfo.BYTE_SIZE, ei.byteData());
    }

    /* packaged */ BeginClient(ByteSequence data) { super(data); }


    /* ----- inspectors ----- */

    public int getResultCode() { return getKey(); }


    /* ----- processing ----- */

    public void process(Main main) {
	if (Globals.sessionState == Globals.SESSION_SHUTDOWN) return;
	if (Globals.sessionState != Globals.SESSION_MSG_BEGIN_CLIENT) {
	    main.serverError();
	    return;
	}
	Message.setSessionKey(getKey());
	switch (getParmint()) {
	    case RC_DENIED:
		main.disconnectNow(
		    "Server is single-user, and already in use."
		);
		break;
	    case RC_OKSINGLE:
		main.waitForRunClient();
		break;
	    case RC_OK:
		main.requestPlayerName();
		break;
	    default:
		main.serverError();
		return;
	}
    }

}
