/* Copyright 1999 by Donald Reble. All rights reserved. */

/* $Id: $ */
/* This class represents an change-password message. */

package cgmud.message;

import cgmud.base.ByteSequence;
import cgmud.main.Globals;
import cgmud.main.Main;


public class ChangePassword extends Message {

    private static final int
	ENTER_OLD = 0,
	ENTER_NEW = 1,
	INCORRECT = 2;

    /* ----- constructors ----- */

    public ChangePassword(String pw) {
	// pass null if the user failed to confirm
	super(Message.CHANGE_PASSWORD, (pw == null) ? INCORRECT : 0,
	    (byte) 0, (short) ((pw == null) ? 0 : PASSWORD_LEN));
	if (pw != null) {
	    appendBlock(PASSWORD_LEN, Crypt.encrypt(pw));
	}
    }

    /* packaged */ ChangePassword(ByteSequence data) { super(data); }


    /* ----- processing ----- */

    public void process(Main main) {
	if (Globals.sessionState == Globals.SESSION_SHUTDOWN) return;
	if (Globals.sessionState < Globals.SESSION_PLAYING) {
	    main.serverError();
	    return;
	}
	switch (getKey()) {
	    case ENTER_OLD:
		main.remakePassword(getFlag(), false);
		break;
	    case ENTER_NEW:
		main.remakePassword(getFlag(), true);
		break;
	    case INCORRECT:
		main.printdiag("Old password incorrect.");
		break;
	    default:
		main.serverError();
		break;
	}
    }

}
