/* Copyright 1999 by Donald Reble. All rights reserved. */

/* $Id: $ */
/* This class represents an input buffer and monitoring thread. */

package cgmud.message;

import java.awt.*;
import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.IOException;
import cgmud.event.MessageEvent;
import cgmud.event.ThreadEndEvent;


public class ThreadedReader extends BufferedInputStream implements Runnable {

    private EventQueue d_queue;
    private Component d_handler;
    private Thread d_thread;
    private int d_threadstate; // 0=running, 1=EOF or manual stop, 2=IOerr


    /* ----- constructors ----- */

    public ThreadedReader(InputStream stream, Component eventhandler) {
	super(stream);
	d_queue = Toolkit.getDefaultToolkit().getSystemEventQueue();
	d_handler = eventhandler;
	d_thread = new Thread(this);
	d_threadstate = 0;
	d_thread.start();
    }


    /* ----- Runnable interface ----- */

    public void run() {
	try {
	    for ( ; ; ) {
		Message msg = Message.receive(this);
		if (msg == null) {
//		    System.err.println("EOF on socket.");
		    break;
		}
		d_queue.postEvent(new MessageEvent(d_handler, msg));
	    }
	    d_threadstate = 1;
	    close();
	} catch (IOException ex) {
	    System.err.println("ThreadedReader IOException: "
			       + ex.toString());
	    d_threadstate = 2;
	}
	d_queue.postEvent(new ThreadEndEvent(d_handler));
    }


    /* ----- Thread delegators ----- */

    public void stop() {
	d_threadstate = 1;
//	System.err.println("ThreadedReader.stop() called.");
	try {
	    close();
	} catch (java.net.SocketException sex) {
	    // ignore - expected
	} catch (IOException ex) {
	    System.err.println("ThreadedReader IOException: "
			       + ex.toString());
	    d_threadstate = 2;
	}
	d_queue.postEvent(new ThreadEndEvent(d_handler));
	d_thread.stop();
    }


    /* ----- inspectors ----- */

    public boolean isRunning() { return d_threadstate == 0; }
    public boolean gotError() { return d_threadstate == 2; }

}
