/* Copyright 1999 by Donald Reble. All rights reserved. */

/* $Id: $ */
/* This class represents a Frame of boxes, built according to commands
   from the server. */

package cgmud.ui;

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import cgmud.base.Cache;
import cgmud.event.*;
import cgmud.main.Globals;


public class BoxFrame extends Frame {

    public static final short
	ALIGN_MIN = HVBoxLayout.ALIGN_MIN,
	ALIGN_MED = HVBoxLayout.ALIGN_MED,
	ALIGN_MAX = HVBoxLayout.ALIGN_MAX;

    public static final int
	PROMPTED_TEXT = 1,
	GENERAL_TEXT  = 2,
	ICON_PANE     = 3,
	ICONED_CANVAS = 4,
	BUTTON_PANEL  = 5;

    private Cache d_uiItems;
    private PromptedText d_defPrompt;
    private GeneralText d_defText;
    private IconPane d_defIcons;


    /* ----- constructor ----- */

    public BoxFrame() {
	super("CGMud client");
	setSize(800, 800); // hack to prevent mostly-off-screen placement
	d_uiItems = new Cache();
	d_uiItems.insertItem(0, this);
	d_defPrompt = null;
	d_defText = null;
	d_defIcons = null;
    }


    /* ----- inspectors ----- */

    public Enumeration getEnumeration() {
	return d_uiItems.elements();
    }

    public Component getBoxComponent(int id) {
	return (Component) d_uiItems.getItem(id);
    }

    public PromptedText getPromptedText(int id) {
	return (PromptedText) d_uiItems.getItem(id);
    }

    public GeneralText getGeneralText(int id) {
	return (GeneralText) d_uiItems.getItem(id);
    }

    public IconPane getIconPane(int id) {
	return (IconPane) d_uiItems.getItem(id);
    }

    public IconedCanvas getIconedCanvas(int id) {
	return (IconedCanvas) d_uiItems.getItem(id);
    }

    public ButtonPanel getButtonPanel(int id) {
	return (ButtonPanel) d_uiItems.getItem(id);
    }

    public PromptedText getDefaultPromptedText() {
	return d_defPrompt;
    }

    public GeneralText getDefaultGeneralText() {
	return d_defText;
    }

    public IconPane getDefaultIconPane() {
	return d_defIcons;
    }


    /* ----- construction methods ----- */

    public void addContainer(int encloser, int id, boolean vertical,
			     short alignment)
    {
	Container enc = (Container) getBoxComponent(encloser);
	if (encloser == 0) {
	    if (id == 0) {
		setLayout(new HVBoxLayout(vertical, alignment, 2));
		return;
	    }
	} else {
	    if (! (enc instanceof OrientedPanel)) {
		throw new IllegalArgumentException("adding to non-container");
	    }
	}
	if (getBoxComponent(id) != null) {
	    throw new IllegalArgumentException("container id already used");
	}
	OrientedPanel pan = new OrientedPanel(vertical, alignment);
	d_uiItems.insertItem(id, pan);
	enc.add(pan);
    }

    public void addComponent(int encloser, int id, int kind,
			     short width, short height)
    {
	Container enc = (Container) getBoxComponent(encloser);
	if ((encloser != 0) && (! (enc instanceof OrientedPanel))) {
	    throw new IllegalArgumentException("adding to non-container");
	}
	if (getBoxComponent(id) != null) {
	    throw new IllegalArgumentException("container id already used");
	}

	Component com;
	switch (kind) {
	    case PROMPTED_TEXT:
		com = new PromptedText(Globals.theFixedFont, width);
		if (d_defPrompt == null) d_defPrompt = (PromptedText) com;
		break;
	    case GENERAL_TEXT:
		com = new GeneralText(Globals.theFixedFont,
		    width, height);
		if (d_defText == null) d_defText = (GeneralText) com;
		break;
	    case ICON_PANE:
		d_defIcons = new IconPane(width, height);
		com = d_defIcons;
		break;
	    case ICONED_CANVAS:
		com = new IconedCanvas(Globals.theFixedFont, width, height);
		com.addMouseListener(Globals.theMain); // %%%
		break;
	    case BUTTON_PANEL:
		com = new ButtonPanel(Globals.theFixedFont, width, height);
		break;
	    default:
		throw new IllegalArgumentException
		    ("invalid component kind");
	}
	d_uiItems.insertItem(id, com);
	enc.add(com);

    }

    public void makeFrame() {
	pack();
	setVisible(true);
	enableEvents(AWTEvent.FOCUS_EVENT_MASK);
	Globals.theIconPane = d_defIcons;
    }


    /* ----- Component overrides ----- */

    public void processEvent(AWTEvent ev) {
	if (ev instanceof LineEvent) {
	    Globals.theMain.processLineEvent((LineEvent) ev);
	} else if (ev instanceof MessageEvent) {
	    Globals.theMain.processMessageEvent((MessageEvent) ev);
	} else if (ev instanceof ThreadEndEvent) {
	    Globals.theMain.processThreadEndEvent((ThreadEndEvent) ev);
	}
    }

}
