/* Copyright 2000 by Donald Reble. All rights reserved. */

/* $Id: $ */
/* This class is a substitute for sun.awt.Orientable.FlowLayout.
   But all borders are the same size. */

package cgmud.ui;

import java.awt.*;


public class HVBoxLayout implements LayoutManager {

    public static final short
	ALIGN_MIN = -1,
	ALIGN_MED =  0,
	ALIGN_MAX =  1;


    boolean d_vertical;
    short d_alignment;
    int d_border;
    boolean d_didlayout;
    Dimension d_size;


    /* ----- constructor ----- */

    public HVBoxLayout(boolean vertical, short alignment, int border) {
	d_vertical = vertical;
	d_alignment = alignment;
	d_border = border;
	d_didlayout = false;
	d_size = new Dimension(0, 0);
    }


    /* ----- mutators ----- */

    public void addLayoutComponent(String name, Component comp) {
	d_didlayout = false;
    }

    public void removeLayoutComponent(Component comp) {
	d_didlayout = false;
    }

    private void m_doLayout(Container parent) {
	if (d_didlayout) return;

	if (d_vertical) {
	    d_size.width = 0;
	    d_size.height = d_border;
	} else {
	    d_size.width = d_border;
	    d_size.height = 0;
	}
	int qu = parent.getComponentCount();
	for (int ix = 0; ix < qu; ix += 1) {
	    Component co = parent.getComponent(ix);
	    Dimension codim = co.getPreferredSize();
	    if (d_vertical) {
		int wid = codim.width + d_border + d_border;
	    	if (d_size.width < wid) d_size.width = wid;
		d_size.height += codim.height + d_border;
	    } else {
		int hei = codim.height + d_border + d_border;
	    	if (d_size.height < hei) d_size.height = hei;
		d_size.width += codim.width + d_border;
	    }
	}

	Insets ins = parent.getInsets();
	d_size.width += ins.left + ins.right;
	d_size.height += ins.top + ins.bottom;

	int pos = d_border;
	if (d_vertical) {
	    pos += ins.top;
	} else {
	    pos += ins.left;
	}
	for (int ix = 0; ix < qu; ix += 1) {
	    Component co = parent.getComponent(ix);
	    Dimension codim = co.getPreferredSize();
	    int just = 0;
	    if (d_vertical) {
		if (d_alignment < ALIGN_MED) {
		    just = d_border + ins.left;
		} else if (d_alignment > ALIGN_MED) {
		    just = d_size.width - codim.width
			 - d_border - ins.right;
		} else {
		    just = (d_size.width - codim.width) / 2;
		}
		co.setBounds(just, pos, codim.width, codim.height);
		pos += codim.height + d_border;
	    } else {
		if (d_alignment < ALIGN_MED) {
		    just = d_border + ins.top;
		} else if (d_alignment > ALIGN_MED) {
		    just = d_size.height - codim.height
			 - d_border - ins.bottom;
		} else {
		    just = (d_size.height - codim.height) / 2;
		}
		co.setBounds(pos, just, codim.width, codim.height);
		pos += codim.width + d_border;
	    }
	}

	d_didlayout = true;
    }

    public void layoutContainer(Container parent) {
	m_doLayout(parent);
    }

    public Dimension minimumLayoutSize(Container parent) {
    	return preferredLayoutSize(parent);
    }

    public Dimension preferredLayoutSize(Container parent) {
	m_doLayout(parent);
	return new Dimension(d_size);
    }

}
