/* Copyright 1999 by Donald Reble. All rights reserved. */

/* $Id: $ */
/* This class represents an icon: a 16x16 bitmap with two colors, one
   transparent and one specified. The constructor takes a Component
   parameter: its peer must already exist. An additional flag is
   associated with the Icon; this is typically used as a
   show/don't-show flag. */

package cgmud.ui;

import java.awt.*;
import java.awt.image.*;


public class Icon {

    private static final byte[] D_STANDARD_PLAYER = {
	(byte) 0x00, (byte) 0x00,
	(byte) 0x00, (byte) 0x00,
	(byte) 0x03, (byte) 0xc0,
	(byte) 0x0c, (byte) 0x30,
	(byte) 0x10, (byte) 0x08,
	(byte) 0x10, (byte) 0x08,
	(byte) 0x26, (byte) 0x64,
	(byte) 0x20, (byte) 0x04,
	(byte) 0x20, (byte) 0x04,
	(byte) 0x24, (byte) 0x24,
	(byte) 0x13, (byte) 0xc8,
	(byte) 0x10, (byte) 0x08,
	(byte) 0x0c, (byte) 0x30,
	(byte) 0x03, (byte) 0xc0,
	(byte) 0x00, (byte) 0x00,
	(byte) 0x00, (byte) 0x00,
    };
    private static final byte[] D_STANDARD_MONSTER = {
	(byte) 0x0f, (byte) 0xe0,
	(byte) 0x78, (byte) 0x3c,
	(byte) 0x40, (byte) 0x04,
	(byte) 0x5c, (byte) 0x74,
	(byte) 0xd4, (byte) 0x56,
	(byte) 0x8c, (byte) 0x62,
	(byte) 0xa0, (byte) 0x0a,
	(byte) 0x93, (byte) 0x92,
	(byte) 0x8a, (byte) 0xa2,
	(byte) 0x80, (byte) 0x02,
	(byte) 0x84, (byte) 0x42,
	(byte) 0xcd, (byte) 0x66,
	(byte) 0x67, (byte) 0xcc,
	(byte) 0x30, (byte) 0x18,
	(byte) 0x1f, (byte) 0xf0,
	(byte) 0x00, (byte) 0x00,
    };

    private Image d_image;
    private boolean d_flag;


    /* ----- constructor ----- */

    public Icon(byte[] bitmap, int colint, Component drawon) {
	byte[] colormap = new byte[6];
	colormap[0] = 0;
	colormap[1] = 0;
	colormap[2] = 0;
	colormap[3] = (byte) (colint >> 16);
	colormap[4] = (byte) (colint >>  8);
	colormap[5] = (byte)  colint;

	byte[] bytemap = new byte[256];
	int mapx = 0;
	for (int bx = 0; bx < 32; bx += 1) {
	    byte by = bitmap[bx];
	    for (int bit = 0x80; bit != 0; bit >>= 1) {
		bytemap[mapx] = (byte) (((by & bit) == 0) ? 0 : 1);
		mapx += 1;
	    }
	}

	IndexColorModel icm = new IndexColorModel
	    (1, 2, colormap, 0, false, 0);
	MemoryImageSource mis = new MemoryImageSource
	    (16, 16, icm, bytemap, 0, 16);
	d_image = drawon.createImage(mis);

	d_flag = false;
    }

    public Icon(boolean monster, int colint, Component drawon) {
	this(monster ? D_STANDARD_MONSTER : D_STANDARD_PLAYER,
	    colint, drawon);
    }


    /* ----- inspectors ----- */

    public Image getImage() { return d_image; }
    public boolean getFlag() { return d_flag; }


    /* ----- mutators ----- */

    public void setFlag(boolean flag) { d_flag = flag; }


}
