/* Copyright 1999 by Donald Reble. All rights reserved. */

/* $Id: $ */
/* This class represents a BackedCanvas with an iconic cursor.
   (Not a Java-built-in cursor.) */

package cgmud.ui;

import java.awt.*;
import java.awt.image.*;
import cgmud.main.Globals;


public class IconedCanvas extends BackedCanvas {

    private static final byte[] D_X_CURSOR = {
	(byte) 0x40, (byte) 0x02, (byte) 0xe0, (byte) 0x07,
	(byte) 0x70, (byte) 0x0e, (byte) 0x38, (byte) 0x1c,
	(byte) 0x1c, (byte) 0x38, (byte) 0x0e, (byte) 0x70,
	(byte) 0x07, (byte) 0xe0, (byte) 0x03, (byte) 0xc0,
	(byte) 0x03, (byte) 0xc0, (byte) 0x07, (byte) 0xe0,
	(byte) 0x0e, (byte) 0x70, (byte) 0x1c, (byte) 0x38,
	(byte) 0x38, (byte) 0x1c, (byte) 0x70, (byte) 0x0e,
	(byte) 0xe0, (byte) 0x07, (byte) 0x40, (byte) 0x02
    };


    private Icon d_icon;
    private byte[] d_iconPattern;
    private int d_iconColor;
    private int d_iconx, d_icony;
    private boolean d_showIcon;


    /* ----- constructor ----- */

    public IconedCanvas(Font font, int width, int height) {
	super(font, width, height);
	d_showIcon = false;
	d_iconx = width / 2;
	d_icony = height / 2;
	d_iconPattern = D_X_CURSOR;
	d_iconColor = 0x00ff0000;
	d_icon = new Icon(d_iconPattern, d_iconColor, this);
    }

    public void addNotify() {
	super.addNotify();
	setBackground(new Color(Globals.thePenSet.getColorint(0)));
	int curcol = Globals.thePenSet.getCurrentPen();
	getBackingGraphics().setColor(
	    new Color(Globals.thePenSet.getColorint(curcol))
	);
    }


    /* ----- cursor control ----- */

    public void setIconPattern(byte[] pattern) {
	d_iconPattern = pattern;
	d_icon = new Icon(d_iconPattern, d_iconColor, this);
	if (d_showIcon) repaint();
    }

    public void setIconColor(int color) {
	d_iconColor = color;
	d_icon = new Icon(d_iconPattern, d_iconColor, this);
	if (d_showIcon) repaint();
    }

    public void moveIcon(int px, int py) {
	d_iconx = px;
	d_icony = py;
	m_fixIconLoc();
	if (d_showIcon) repaint();
    }

    public void moveIconRelative(int dx, int dy) {
	d_iconx += dx;
	d_icony += dy;
	m_fixIconLoc();
	if (d_showIcon) repaint();
    }

    private void m_fixIconLoc() {
	Dimension size = getSize();
	if (d_iconx < 0) d_iconx = 0;
	if (d_icony < 0) d_icony = 0;
	if (d_iconx > size.width) d_iconx = size.width;
	if (d_icony > size.height) d_iconx = size.height;
    }

    public void showIcon(boolean show) {
	if (d_showIcon != show) {
	    d_showIcon = show;
	    repaint();
	}
    }


    /* ----- graphics methods ----- */

    // inherit public Graphics getBackingGraphics();

    // inherit public void replaceImage(Image im);

    public void paint(Graphics gr) {
	super.paint(gr);
	if (d_showIcon && (d_icon != null)) {
	    gr.drawImage(d_icon.getImage(), d_iconx, d_icony, this);
	}
    }

    public void update(Graphics gr) {
	paint(gr);
    }

}
