/* Copyright 1999 by Donald Reble. All rights reserved. */

/* $Id: $ */
/* This class represents the drawing pens of cgMUD.

   The colour model uses 252 colours, the cartesian product of six red
   values, seven green values, and six blue values. Such values include
   0x00 and 0xff, and are (almost) evenly distributed between those.
   One specifies such a colour as 6*7*red + 6*green + blue.
   The last four colors are greys. */

package cgmud.ui;

import java.awt.Color;
import java.awt.Component;
import java.util.Enumeration;
import cgmud.main.Globals;


public class PenSet {

    public static final int PEN_QUAN_LG = 8;
    public static final int PEN_QUAN = 256;

    private static final int[] D_DEFAULT_COLORS;

    private int[] d_penColors;
    private int d_currentPen;

    static {
	D_DEFAULT_COLORS = new int[PEN_QUAN];
	int idx = 0;
	for (int rd = 0; rd < 6; rd += 1) {
	    int rdval = ((255 * rd) / 5) << 16;
	    for (int gr = 0; gr < 7; gr += 1) {
	    	int grval = ((255 * gr) / 6) << 8;
		for (int bl = 0; bl < 6; bl += 1) {
		    int blval = (255 * bl) / 5;
		    D_DEFAULT_COLORS[idx] =
		    	0xff000000 + rdval + grval + blval;
		    idx += 1;
		}
	    }
	}
	D_DEFAULT_COLORS[252] = 0xff444444;
	D_DEFAULT_COLORS[253] = 0xff888888;
	D_DEFAULT_COLORS[254] = 0xffcccccc;
	D_DEFAULT_COLORS[255] = 0xffffffff;
    }


    /* ----- constructor ----- */

    public PenSet() {
	d_penColors = new int[PEN_QUAN];
	m_initPens();
	d_currentPen = 0;
    }


    /* ----- other methods ----- */

    private void m_initPens() {
	System.arraycopy(D_DEFAULT_COLORS, 0, d_penColors, 0, PEN_QUAN);
    }

    private void m_prepareDrawing() {
	Color co = new Color(d_penColors[d_currentPen]);
	for (Enumeration uis = Globals.theFrame.getEnumeration();
	     uis.hasMoreElements();
	    )
	{
	    try {
		BackedCanvas bc = (BackedCanvas) uis.nextElement();
		bc.getBackingGraphics().setColor(co);
		if (d_currentPen == 0) {
		    bc.setBackground(co);
		}
	    } catch (ClassCastException cx) {
		// It's a different component. Do nothing.
	    }
	}
    }

    public void resetPens() {
	m_initPens();
	m_prepareDrawing();
    }

    public int getCurrentPen() { return d_currentPen; }

    public int getColorint(int pen) {
	return d_penColors[pen];
    }

    public void setCurrentPen(int pen) {
	if ((pen >= 0) && (pen < PEN_QUAN)) {
	    d_currentPen = pen;
	    m_prepareDrawing();
	}
    }

    public void setPen(int pen, int rgb) {
	d_penColors[pen] = rgb | 0xff000000;
	if (pen == d_currentPen) {
	    m_prepareDrawing();
	}
    }

}
