/* Copyright 1999 by Donald Reble. All rights reserved. */

/* $Id: $ */
/* This class takes typed lines from the user. It interprets keypad
   strokes specially. */

package cgmud.ui;

import java.awt.*;
import java.awt.event.*;
import cgmud.event.LineEvent;
import cgmud.main.Globals;
import cgmud.message.RawKey;


public class PromptedText extends Panel implements KeyListener, FocusListener {

    Label d_prompt;
    TextField d_input;
    EventQueue d_evq;
    int d_panelCols, d_charWidth;
    Color d_foreground;
    boolean d_firstPaint;


    /* ----- constructors ----- */

    public PromptedText(Font font, int cols) {
	super();
	setLayout(null);
	setFont(font);

	Toolkit tk = Toolkit.getDefaultToolkit();
	d_evq = tk.getSystemEventQueue();

	FontMetrics fm = tk.getFontMetrics(font);
	d_panelCols = cols;
	int prmtwid = cols / 3;
	if (prmtwid > 30) prmtwid = 30;

//	d_charWidth = fm.getMaxAdvance();     wrong on Linux/JDK 1.3 -cg
	d_charWidth = fm.charWidth('M');
	int promptwid = prmtwid * d_charWidth;
	int panelwid = cols * d_charWidth;

	d_prompt = new Label("", Label.CENTER);
	    d_prompt.setBounds(0, 0, promptwid, 30);
	    add(d_prompt);
	    d_prompt.addKeyListener(this);
	d_input = new TextField();
	    d_input.setBounds(promptwid, 0, panelwid - promptwid, 30);
	    add(d_input);
	    d_input.addKeyListener(this);
	    d_input.addFocusListener(this);
	setSize(panelwid, 30);
	d_input.setBackground(Color.white);
	d_foreground = d_input.getForeground();
	d_firstPaint = true;
    }


    /* ----- other methods ----- */

/* Good code, I think, but broken. -djr
    public void setBlanking(boolean blank) {
	if (blank) {
	    d_typeArea.setEchoChar('*');
	} else {
	    d_typeArea.setEchoChar('\0');
	}
    }
*/

    public void setBlanking(boolean blank) {
	d_input.setForeground(blank ? d_input.getBackground() : d_foreground);
    }

    public String getText() {
	return d_input.getText();
    }

    public String getPrompt() {
	return d_prompt.getText();
    }

    public void setPrompt(String prompt) {
	int prmptlen = prompt.length();
	int prmptwid = prmptlen * d_charWidth;
	int inputlen = d_panelCols - prmptlen;
	d_prompt.setBounds(0, 0, prmptwid, 30);
	d_input.setColumns(inputlen);
	d_input.setBounds(prmptwid, 0, inputlen * d_charWidth, 30);
	d_prompt.setText(prompt);
	repaint();
	d_input.requestFocus();
    }


    /* ----- Component overrides ----- */

    public void paint(Graphics g) {
	// Do this so that the input area is the focus when the client
	// first starts up.
	if (d_firstPaint) {
	    d_firstPaint = false;
	    d_input.requestFocus();
	}
    }


    /* ----- KeyListener interface ----- */

    public void keyReleased(KeyEvent ke) { }
    public void keyTyped(KeyEvent ke) { }

    public void keyPressed(KeyEvent ke) {
	int kc = ke.getKeyCode();
	switch (kc) {
	    case KeyEvent.VK_ENTER:
		d_evq.postEvent(new LineEvent(Globals.theFrame,
		    Globals.theDefaultPrompter.getText(), true));
		d_input.setText("");
		setBlanking(false);
		ke.consume();
		return;

	    case KeyEvent.VK_NUMPAD1:
		Globals.theMain.handleRawKey(RawKey.RAWKEY_DOWNLEFT);
		ke.consume();
		return;
	    case KeyEvent.VK_NUMPAD2:
	    case KeyEvent.VK_DOWN:
		Globals.theMain.handleRawKey(RawKey.RAWKEY_DOWN);
		ke.consume();
		return;
	    case KeyEvent.VK_NUMPAD3:
		Globals.theMain.handleRawKey(RawKey.RAWKEY_DOWNRIGHT);
		ke.consume();
		return;
	    case KeyEvent.VK_NUMPAD4:
	    case KeyEvent.VK_LEFT:
		Globals.theMain.handleRawKey(RawKey.RAWKEY_LEFT);
		ke.consume();
		return;
	    case KeyEvent.VK_NUMPAD5:
		Globals.theMain.handleRawKey(RawKey.RAWKEY_CENTER);
		ke.consume();
		return;
	    case KeyEvent.VK_NUMPAD6:
	    case KeyEvent.VK_RIGHT:
		Globals.theMain.handleRawKey(RawKey.RAWKEY_RIGHT);
		ke.consume();
		return;
	    case KeyEvent.VK_NUMPAD7:
		Globals.theMain.handleRawKey(RawKey.RAWKEY_UPLEFT);
		ke.consume();
		return;
	    case KeyEvent.VK_NUMPAD8:
	    case KeyEvent.VK_UP:
		Globals.theMain.handleRawKey(RawKey.RAWKEY_UP);
		ke.consume();
		return;
	    case KeyEvent.VK_NUMPAD9:
		Globals.theMain.handleRawKey(RawKey.RAWKEY_UPRIGHT);
		ke.consume();
		return;
	    case KeyEvent.VK_ADD:
		Globals.theMain.handleRawKey(RawKey.RAWKEY_PLUS);
		ke.consume();
		return;
	    case KeyEvent.VK_SUBTRACT:
		Globals.theMain.handleRawKey(RawKey.RAWKEY_MINUS);
		ke.consume();
		return;
	    case KeyEvent.VK_HELP:
		Globals.theMain.handleRawKey(RawKey.RAWKEY_HELP);
		ke.consume();
		return;
	}
    }


    /* ----- FocusListener interface ----- */

    public void focusGained(FocusEvent evt) { }

    public void focusLost(FocusEvent evt) {
	// Only do this if user asks for it. It messes up badly on
	// Windows, apparantly. Possibly relates to whether or not the
	// host GUI is doing autofocus type things.
	if (Globals.refocus) {
	    d_input.requestFocus();
	}
    }

}
